<?php

namespace CRM\Factory;

use Doctrine\ORM\EntityManager;
use CRM\Traits\TQuery;

class CrmEtapaProspectFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getProspectSteps($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder
                ->select([
                    'NRSEQETAPA',
                    'NMETAPA',
                    'ETORDEM',
                    'IDATIVO'
                ])
                ->from('CRM_CDETAPAS')
                ->where($queryBuilder->expr()->eq('NRORG', ':NRORG'))
                ->orderBy('ETORDEM', 'ASC');

            $queryBuilder->setParameter('NRORG', $params['NRORG']);
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function insertProspectStep($params) {
        try {
            $sql = "INSERT INTO CRM_CDETAPAS
                                (
                                    NRSEQETAPA,
                                    NMETAPA,
                                    IDATIVO,
                                    ETORDEM,
                                    NRORG,
                                    CRM_NRORGTRAB,
                                    CDOPERINCLUSAO,
                                    DTINCLUSAO,
                                    DTULTATU
                                )
                        VALUES (
                                (SELECT MAX (NRSEQETAPA) +1 FROM CRM_CDETAPAS),
                                :NMETAPA,
                                :IDATIVO,
                                :ETORDEM,
                                :NRORG,
                                :CRM_NRORGTRAB,
                                :CDOPERINCLUSAO,
                                SYSDATE,
                                SYSDATE
                                )";
    
            $this->connection->executeStatement($sql, $params);
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function editProspectStep($params)
    {        
        try {
            $sql = "UPDATE CRM_CDETAPAS
                    SET NMETAPA = :NMETAPA, 
                    ETORDEM = :ETORDEM, 
                    IDATIVO = :IDATIVO,
                    DTULTATU = SYSDATE
                    WHERE NRORG = :NRORG
                    AND   CRM_NRORGTRAB = :CRM_NRORGTRAB
                    AND   NRSEQETAPA = :NRSEQETAPA";
    
            $this->connection->executeStatement($sql, $params);
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getStepsAmount()
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder->select([
                "COUNT(NRSEQETAPA)",
            ])
                ->from('CRM_CDETAPAS', 'CRM_CDETAPAS')
                ->where("IDATIVO = 'S'");
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function saveStepsOrder($rows) {}
}
