<?php

namespace CRM\Factory;

use Doctrine\ORM\EntityManager;
use CRM\Traits\TQuery;

class CrmPerfilDeAtividadeFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getCrmPerfilDeAtividade()
    {
        $sql = "SELECT * FROM CRM_PERFILATIV ORDER BY NRSEQPERFATIV ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function insertCrmPerfilDeAtividade( $params )
    {
        $sql = "INSERT INTO 
                CRM_PERFILATIV
                (
                    NRSEQPERFATIV,
                    DESCPEFATIV,
                    IDDASHPADR,
                    NRORG,
                    IDATIVO,
                    DTINCLUSAO,
                    NRORGINCLUSAO,
                    CDOPERINCLUSAO,
                    IDEXIBEPAINELVENDAS,
                    IDEXIBEPAINELATEND
                )
                VALUES
                (
                    :NRSEQPERFATIV,
                    :DESCPEFATIV,
                    :IDDASHPADR,
                    :NRORG,
                    :IDATIVO,
                    SYSDATE,
                    :NRORGINCLUSAO,
                    :CDOPERINCLUSAO,
                    :IDEXIBEPAINELVENDAS,
                    :IDEXIBEPAINELATEND 
                )";
        $this->connection->executeStatement($sql, $params);
       
    }


    public function editCrmPerfilDeAtividade( $params )
    {
        
        $sql = "UPDATE CRM_PERFILATIV
                SET 
                    DESCPEFATIV                = :DESCPEFATIV,
                    NRORG                      = :NRORG,
                    NRORGULTATU                = :NRORGULTATU,  
                    CDOPERULTATU               = :CDOPERULTATU,  
                    IDATIVO                    = :IDATIVO,
                    IDDASHPADR                 = :IDDASHPADR,
                    DEXIBEPAINELVENDAS         = :DEXIBEPAINELVENDAS,
                    IDEXIBEPAINELATEND         = :IDEXIBEPAINELATEND,
                    DTULTATU                   = SYSDATE
                WHERE NRSEQPERFATIV              = :NRSEQPERFATIV";
        $this->connection->executeStatement($sql, $params);

    }

    public function checkExistingPerfilDeAtividade($params)
    {
        $sql = "SELECT
                    COUNT(*) AS VALIDA
                FROM
                    CRM_PERFILATIV
                WHERE
                       UPPER (DESCPEFATIV) = UPPER (:DESCPEFATIV)
                      AND NRSEQPERFATIV <>NVL( :NRSEQPERFATIV, -1)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]["VALIDA"];
    }

    public function deleteCrmPerfilDeAtividade($params)
    {  
        $sql = "DELETE FROM CRM_PERFILATIV
                WHERE NRSEQPERFATIV         = :NRSEQPERFATIV
                    AND NRORG               = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
    }
    
    public function getCrmPerfilDeAtividadeOperador($params)
    {
        $sql = "SELECT PERFIOPER.*,
                        PERFIL.DESCPEFATIV,
                        OPER.NMOPERADOR
                FROM CRM_PERFIOPER PERFIOPER
                LEFT JOIN OPERADOR OPER
                    ON OPER.CDOPERADOR = PERFIOPER.CDOPERADOR
                    AND OPER.NRORG     = PERFIOPER.NRORG
                LEFT JOIN CRM_PERFILATIV PERFIL
                    ON PERFIL.NRSEQPERFATIV = PERFIOPER.NRSEQPERFATIV
                    AND PERFIL.NRORG        = PERFIOPER.NRORG
                    WHERE PERFIL.NRSEQPERFATIV = :NRSEQPERFATIV";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getCrmPerfilDeAtividadeOperadores($params)
    {
        $sql = "SELECT 
                    CDOPERADOR,
                    NMOPERADOR,
                    DSEMAILOPER,
                    IDOPERATIVO,
                    NRORG
                FROM OPERADOR
                WHERE NRORG = :NRORG";
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getCrmPerfilOperador($params)
    {
        $sql = "SELECT CDOPERADOR,
  NMOPERADOR,
  NRORG
FROM OPERADOR
WHERE ((:IDTPOPERADOR    = 'R'
AND OPERADOR.CDOPERADOR IN
  (SELECT CDOPERADOR FROM CRM_OPERREVENDA WHERE NRREVENDA = :NRREVENDA
  ))
OR :IDTPOPERADOR <> 'R')";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }


    public function getCrmPerfilModalMetas()
    {
        $sql = "SELECT NRMETAS,
                        NMMETA,
                        CDTIPOMETA,
                        VRMINMETA,
                        IDATIVO,
                        NRORG,
                        NRMETAS
                        || ' - '
                        || NMMETA 
                        || ' - '
                        || VRMINMETA
                        || ' - '
                        || CDTIPOMETA
                        || ' - '
                        || IDATIVO
                        || ' - '
                        || NRORG
                AS DESCRICAO
                FROM CRM_METAS
                ORDER BY NMMETA ASC";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getCrmPerfilDeAtividadeMetas($params)
    {
        $sql = "SELECT PERFIOPER.*,
                    PERFIL.DESCPEFATIV,
                    OPER.NMOPERADOR,
                    METAS.NRMETAS,
                    METAS.NMMETA,
                    METAS.CDTIPOMETA,
                    METAS.VRMINMETA
                    FROM CRM_PERFIOPER PERFIOPER
                    LEFT JOIN OPERADOR OPER
                    ON OPER.CDOPERADOR = PERFIOPER.CDOPERADOR
                    AND OPER.NRORG = PERFIOPER.NRORG
                    LEFT JOIN CRM_PERFILATIV PERFIL
                    ON PERFIL.NRSEQPERFATIV = PERFIOPER.NRSEQPERFATIV
                    AND PERFIL.NRORG = PERFIOPER.NRORG
                    LEFT JOIN CRM_OPERMETA OPMETA
                    ON OPMETA.CDOPERADOR = PERFIOPER.CDOPERADOR
                    AND OPMETA.NRORG = PERFIOPER.NRORG
                    LEFT JOIN CRM_METAS METAS
                    ON METAS.NRMETAS = OPMETA.NRMETAS
                    AND METAS.NRORG = PERFIOPER.NRORG
                    WHERE PERFIL.NRSEQPERFATIV = :NRSEQPERFATIV";
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function insertCrmPerfilMeta( $params )
    {
        $sql = "INSERT
                    INTO CRM_OPERMETA
                    (
                        NROPERMETA,
                        NRORG,
                        NRMETAS,
                        CDOPERADOR,
                        NRSEQPERFATIV,
                        DTINCLUSAO,
                        NRORGINCLUSAO,
                        CDOPERINCLUSAO
                    )
                    VALUES
                    (
                        :NROPERMETA,
                        :NRORG, 
                        :NRMETAS,
                        :CDOPERADOR, 
                        :NRSEQPERFATIV,
                        SYSDATE,
                        :NRORGINCLUSAO,
                        :CDOPERINCLUSAO)";
        $this->connection->executeStatement($sql, $params);
       
    }

    public function editCrmPerfilMeta($params)
{
    // Exemplo de lógica para editar o perfil meta, de acordo com seus requisitos
    $sql = "UPDATE CRM_OPERMETA 
                SET
                NRSEQPERFATIV   = :NRSEQPERFATIV,
                NRMETAS         = :NRMETAS,
                CDOPERADOR      = :CDOPERADOR,
                NRORG           = :NRORG,
                NRORGINCLUSAO   = :NRORGINCLUSAO,
                CDOPERINCLUSAO  = :CDOPERINCLUSAO,
                DTINCLUSAO      = :DTINCLUSAO
            WHERE NROPERMETA    = :NROPERMETA";

    $stmt = $this->connection->prepare($sql);
    $stmt->execute($params);
}

}