<?php

namespace CRM\Service;

use CRM\Factory\CrmEtapaProspectFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class CrmEtapaProspectService
{

    protected $CrmEtapaProspectFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(Environment $Environment, NewCode $newCode, EntityManager $entityManager, CrmEtapaProspectFactory $CrmEtapaProspectFactory)
    {
        $this->CrmEtapaProspectFactory         = $CrmEtapaProspectFactory;
        $this->Environment                        = $Environment;
        $this->newCode                            = $newCode;
        $this->connection                         = $entityManager->getConnection();
    }

    public function getProspectSteps()
    {        
        try {
            $nrorg = $this->Environment->getNrorg();
            $params = array(
                "NRORG" => $nrorg,
            );
            return $this->CrmEtapaProspectFactory->getProspectSteps($params);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveProspectSteps($rows)
    {
        $this->connection->beginTransaction();
        $nrorg = $this->Environment->getNrorg();
        $operador = $this->Environment->getCdOperador();
        
        try {
            foreach ($rows["DRG"] as $row) {
                if ($row["NRSEQETAPA"] == 0) {
                    print_r('É NOVO');
                    $cdNrEtapa = $this->newCode->getCodeByTableName("CRM_CDETAPAS", $nrorg, 1, 5, false);

                    $params = array(
                        "NRSEQETAPA"        => $cdNrEtapa,
                        "NMETAPA"           => $row["NMETAPA "],
                        "NRETPANTE"         => $row["NRETPANTE"],
                        "NRETPPROX"         => $row["NRETPPROX"],
                        "IDATIVO"           => $row["IDATIVO"],
                        "NRORG"             => $nrorg,
                        "CRM_NRORGTRAB"     => $nrorg,
                        "CDOPERINCLUSAO"    => $operador,
                    );

                    $this->CrmEtapaProspectFactory->insertProspectStep($params);
                } else {
                    print_r('É VELHO');
                    $params = array(
                        "NRSEQETAPA"        => $row['NRSEQETAPA'],
                        "NMETAPA"           => $row["NMETAPA "],
                        "NRETPANTE"         => $row["NRETPANTE"],
                        "NRETPPROX"         => $row["NRETPPROX"],
                        "IDATIVO"           => $row["IDATIVO"],
                        "NRORG"             => $nrorg,
                        "CRM_NRORGTRAB"     => $nrorg,
                        "CDOPERINCLUSAO"    => $operador,
                    );

                    $this->CrmEtapaProspectFactory->editProspectStep($params);
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function getStepsAmount()
    {
        try {
            return $this->CrmEtapaProspectFactory->getStepsAmount();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveStepsOrder($rows)
    { 
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            foreach ($rows["DRG"] as $row) {
                $params = array(
                    "NRSEQETAPA"        => $row["NRSEQETAPA"],
                    "NMETAPA"           => $row["NMETAPA"],
                    "IDATIVO"           => $row["IDATIVO"],
                    "ETORDEM"           => $row["ETORDEM"],
                    "NRORG"             => $nrorg,
                    "CRM_NRORGTRAB"     => $nrorg,
                    "CDOPERINCLUSAO"    => $operador,
                );

                if ($row["NRSEQETAPA"] == 0) $this->CrmEtapaProspectFactory->insertProspectStep($params);
                else $this->CrmEtapaProspectFactory->editProspectStep($params);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
