<?php

namespace CRM\Service;

use CRM\Factory\CrmNovoDashboardFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Guzzle\Http\Message\Request;

class CrmNovoDashboardService
{

    protected $DashboardFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(Environment $Environment, NewCode $newCode, EntityManager $entityManager, CrmNovoDashboardFactory $DashboardFactory)
    {
        $this->DashboardFactory             = $DashboardFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    public function getTasks()
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $user = $this->Environment->getCurrentUserId();

            $params = array(
                "NRORG" => $nrorg,
                "NROPERADOR" => $user
            );
            return $this->DashboardFactory->getTasks($params);
        } catch (\Error $e) {
            throw $e;
        }
    }

    public function getTasksCards($periodo)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $user = $this->Environment->getCurrentUserId();

            $params = array(
                "NRORG" => $nrorg,
                "NROPERADOR" => $user
            );
            return $this->DashboardFactory->getTasksCards($params, $periodo);
        } catch (\Error $e) {
            throw $e;
        }
    }

    public function getOportunities()
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $user = $this->Environment->getCurrentUserId();
            $params = array(
                "NRORG" => $nrorg,
                "NROPERADOR" => $user
            );
            return $this->DashboardFactory->getOportunities($params);
        } catch (\Error $e) {
            throw $e;
        }
    }

    public function getLeads($request)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $user = $this->Environment->getCurrentUserId();
            $params = array(
                "NRORG" => $nrorg,
                "CRM_NRORGTRAB" => $nrorg,
                "NRPARCPROPRI" => $user
            );
            $leads = $this->DashboardFactory->getLeads($params);

            $incio = $this->DashboardFactory->getBeginingStep($params);
            $etapaInicial = $incio['data'][0];

            foreach ($leads["data"] as $key => $row) {
                if (is_null($row['NRETP_ATUAL'])) {
                    $leads["data"][$key]['NRETP_ATUAL'] = $etapaInicial['NRSEQETAPA'];
                    $leads["data"][$key]['ORDETP_ATUAL'] = $etapaInicial['ETORDEM'];
                    $leads["data"][$key]['NMETP_ATUAL'] = $etapaInicial['NMETAPA'];
                }

                $parameter = array(
                    "ETORDEM_ATUAL" => $leads["data"][$key]['ORDETP_ATUAL'],
                    "NRORG" => $nrorg,
                    "CRM_NRORGTRAB" => $nrorg,
                );

                if (is_null($row['NRETP_PROX'])) {
                    $proxEtapa = $this->DashboardFactory->getNextStep($parameter);
                    $leads["data"][$key]['NRETP_PROX'] = $proxEtapa['data'][0]['NRSEQETAPA'];
                    $leads["data"][$key]['NMETP_PROX'] = $proxEtapa['data'][0]['NMETAPA'];
                }
            }

            return ($leads);
        } catch (\Error $e) {
            throw $e;
        }
    }

    public function getLeadProgress($request)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $params = array(
                "NRORG" => $nrorg,
                "CRM_NRORGTRAB" => $nrorg,
                "NRPARCNEGOCIO" => $request['NRPARCNEGOCIO']
            );
            return $this->DashboardFactory->getLeadProgress($params);
        } catch (\Error $e) {
            throw $e;
        }
    }

    public function getLeadProgressHist($request)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $params = array(
                "NRORG" => $nrorg,
                "CRM_NRORGTRAB" => $nrorg,
                "NRPARCNEGOCIO" => $request['NRPARCNEGOCIO']
            );
            if(!count($this->DashboardFactory->getEtapaProspect($params)['data'])) {
                $this->DashboardFactory->createLeadStep($params);
            }
            
            $progressHist = $this->DashboardFactory->getLeadProgressHist($params);
            if (count($progressHist['data'])) {
                $progress = $progressHist['data'][0];

                $obj = array();

                if ($progress['ANT_NRSEQETAPA']) {
                    $obj[] = array(
                        "id" => $progress['ANT_NRSEQETAPA'],
                        "name" => $progress['ANT_NMETAPA'],
                        "column" => '1',
                        "text" => $progress['ANT_NMETAPA']
                    );
                }

                $obj[] = array(
                    "id" => $progress['NRSEQETAPA'],
                    "name" => $progress['NMETAPA'],
                    "column" => '2',
                    "text" => $progress['NMETAPA']
                );

                if ($progress['PROX_NRSEQETAPA']) {
                    $obj[] = array(
                        "id" => $progress['PROX_NRSEQETAPA'],
                        "name" => $progress['PROX_NMETAPA'],
                        "column" => '3',
                        "text" => $progress['PROX_NMETAPA']
                    );
                }

                $progressHist['data'] = $obj;
            } 

            return $progressHist;
        } catch (\Error $e) {
            throw $e;
        }
    }

    public function updateProgess($request)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $user = $this->Environment->getCurrentUserId();
            $params = array(
                "NRORG" => $nrorg,
                "CRM_NRORGTRAB" => $nrorg,
                "CDOPERULTATU" => $user,
                "NRPARCNEGOCIO"  => $request['NRPARCNEGOCIO']
            );

            return $this->DashboardFactory->updateProgess($params);
        } catch (\Error $e) {
            throw $e;
        }
    }

    public function getLosts($request)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $user = $this->Environment->getCurrentUserId();
            $params = array(
                "NRORG" => $nrorg,
                "NROPERADOR" => $user
            );

            return $this->DashboardFactory->getLosts($params);
        } catch (\Error $e) {
            throw $e;
        }
    }

    public function getLostsChart($request)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $user = $this->Environment->getCurrentUserId();
            $params = array(
                "NRORG" => $nrorg,
                "NROPERADOR" => $user
            );

            return $this->DashboardFactory->getLostsChart($params);
        } catch (\Error $e) {
            throw $e;
        }
    }
}
