import { Utilities } from '@/util/Utilities';
import {
  LoadingService,
  GridEditable,
  Button,
} from '@zeedhi/common';
import { Http, IEventParam } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { DragGrid } from '@zeedhi/zd-drag-grid';

export class CrmEtapaProspectController {
  public canEditRow({ row }: any) {
    return row['.__added_row'] === true;
  }

  public addNewRow({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const newRow = {
      NRSEQETAPA: 0,
      NMETAPA: 'Nova Etapa',
      ETORDEM: 0,
      IDATIVO: 'S',
    };
    grid.editingNewRows = true;
    grid.addNewRow(newRow, 'start');
  }

  public openFilter() {
    console.warn('open!');
  }

  public async saveStep({ component, event }: IEventParam<TekGrid>) {
    event?.preventDefault();
    console.warn(component.getEditedRows());
    if (Utilities.validateGrid(component)) {
      LoadingService.show();
      Http.post('/saveProspectStep', {
        DRG: component.getEditedRows(),
      }).then(() => {
        component.datasource.get();
        Utilities.showSuccessMessage();
      }).catch((error: any) => {
        error.showErrorMessage();
      }).finally(() => {
        component.cancelEditedRows();
        LoadingService.hide();
      });
    }
  }

  public messageDelete() {
    console.warn('delete!');
  }

  public editSelect({ row }: any) {
    console.warn(row);
  }

  public editedRows = {};

  public onDragEnd({ component }: IEventParam<DragGrid>) {
    // const titles = component.getEditedRows().map((row) => row.NRSEQETAPA).join(', ');
    const titles = component.getEditedRows();
    console.warn(titles);
    this.editedRows = titles;
  }

  public async saveEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as unknown as DragGrid;
    grid.saveEditedRows();
    console.warn(grid.getEditedRows());

    await Http.post('/saveStepsOrder', {
      DRG: grid.getEditedRows(),
    }).then(() => {
      grid.datasource.get();
      Utilities.showSuccessMessage();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      grid.cancelEditedRows();
      LoadingService.hide();
    });
  }

  public cancelEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as unknown as DragGrid;
    grid.cancelEditedRows();
  }
}
